"""
A set of useful constants from Rocket League in a normal soccar match.

Lengths are measured in Unreal Units (uu), the in-game unit of measurement, equivalent to 1 cm.
"""

# Field dimensions
SIDE_WALL_X = 4096  # +/-
BACK_WALL_Y = 5120  # +/-
CEILING_Z = 2044
BACK_NET_Y = 6000  # +/-
CORNER_CATHETUS_LENGTH = 1152  # uu
RAMP_HEIGHT = 256  # uu

# Goal dimensions
GOAL_HEIGHT = 642.775  # uu
GOAL_CENTER_TO_POST = 892.755  # uu

# Goal locations
ORANGE_GOAL_CENTER = (0, BACK_WALL_Y, GOAL_HEIGHT / 2)
BLUE_GOAL_CENTER = (0, -BACK_WALL_Y, GOAL_HEIGHT / 2)

ORANGE_GOAL_BACK = (0, BACK_NET_Y, GOAL_HEIGHT / 2)  # Often more useful than center
BLUE_GOAL_BACK = (0, -BACK_NET_Y, GOAL_HEIGHT / 2)

ORANGE_GOAL_TOP_LEFT = (GOAL_CENTER_TO_POST, BACK_WALL_Y, GOAL_HEIGHT)
BLUE_GOAL_TOP_LEFT = (-GOAL_CENTER_TO_POST, -BACK_WALL_Y, GOAL_HEIGHT)

ORANGE_GOAL_TOP_RIGHT = (-GOAL_CENTER_TO_POST, BACK_WALL_Y, GOAL_HEIGHT)
BLUE_GOAL_TOP_RIGHT = (GOAL_CENTER_TO_POST, -BACK_WALL_Y, GOAL_HEIGHT)

ORANGE_GOAL_BOTTOM_LEFT = (GOAL_CENTER_TO_POST, BACK_WALL_Y, 0)
BLUE_GOAL_BOTTOM_LEFT = (-GOAL_CENTER_TO_POST, -BACK_WALL_Y, 0)

ORANGE_GOAL_BOTTOM_RIGHT = (-GOAL_CENTER_TO_POST, BACK_WALL_Y, 0)
BLUE_GOAL_BOTTOM_RIGHT = (GOAL_CENTER_TO_POST, -BACK_WALL_Y, 0)

ORANGE_FIELD_TOP_LEFT = (SIDE_WALL_X, BACK_WALL_Y, CEILING_Z)
BLUE_FIELD_TOP_LEFT = (-SIDE_WALL_X, -BACK_WALL_Y, CEILING_Z)

ORANGE_FIELD_TOP_RIGHT = (-SIDE_WALL_X, BACK_WALL_Y, CEILING_Z)
BLUE_FIELD_TOP_RIGHT = (SIDE_WALL_X, -BACK_WALL_Y, CEILING_Z)

ORANGE_FIELD_BOTTOM_LEFT = (SIDE_WALL_X, BACK_WALL_Y, 0)
BLUE_FIELD_BOTTOM_LEFT = (-SIDE_WALL_X, -BACK_WALL_Y, 0)

ORANGE_FIELD_BOTTOM_RIGHT = (-SIDE_WALL_X, BACK_WALL_Y, 0)
BLUE_FIELD_BOTTOM_RIGHT = (SIDE_WALL_X, -BACK_WALL_Y, 0)

GOAL_THRESHOLD = 5215.5  # uu, magnitude of y position required for ball to be inside goal

# Time
TICKS_PER_SECOND = 120
SMALL_PAD_RECHARGE_SECONDS = 4
BIG_PAD_RECHARGE_SECONDS = 10
DEMO_RESPAWN_SECONDS = 3
BOOST_CONSUMPTION_RATE = 33.3  # per second

# Acceleration
BOOST_ACCELERATION = 991.666  # uu/s^2
GRAVITY = 650  # uu/s^2 downwards

# Sizes
BALL_RADIUS = 91.25  # uu
BALL_RESTING_HEIGHT = 93.15  # uu
UNREAL_UNITS_PER_METER = 100  # E.g. 1 meter = 100 uu

# Masses
CAR_MASS = 180  # Arbitrary unit
BALL_MASS = 30

# Speed limits
BALL_MAX_SPEED = 6000  # uu/s
CAR_MAX_SPEED = 2300  # uu/s
SUPERSONIC_THRESHOLD = 2200  # uu/s
CAR_MAX_ANG_VEL = 5.5  # rad/s

# Teams
BLUE_TEAM = 0
ORANGE_TEAM = 1

# Hitboxes
OCTANE = 0
DOMINUS = 1
PLANK = 2
BREAKOUT = 3
HYBRID = 4
MERC = 5

# Actions
NUM_ACTIONS = 8
THROTTLE, STEER, PITCH, YAW, ROLL, JUMP, BOOST, HANDBRAKE = range(8)
DOUBLEJUMP_MAX_DELAY = 1.25
FLIP_TORQUE_TIME = 0.65
JUMP_MAX_TIME = 0.2

# Boost pads
BOOST_LOCATIONS = (
    (0.0, -4240.0, 70.0),
    (-1792.0, -4184.0, 70.0),
    (1792.0, -4184.0, 70.0),
    (-3072.0, -4096.0, 73.0),
    (3072.0, -4096.0, 73.0),
    (- 940.0, -3308.0, 70.0),
    (940.0, -3308.0, 70.0),
    (0.0, -2816.0, 70.0),
    (-3584.0, -2484.0, 70.0),
    (3584.0, -2484.0, 70.0),
    (-1788.0, -2300.0, 70.0),
    (1788.0, -2300.0, 70.0),
    (-2048.0, -1036.0, 70.0),
    (0.0, -1024.0, 70.0),
    (2048.0, -1036.0, 70.0),
    (-3584.0, 0.0, 73.0),
    (-1024.0, 0.0, 70.0),
    (1024.0, 0.0, 70.0),
    (3584.0, 0.0, 73.0),
    (-2048.0, 1036.0, 70.0),
    (0.0, 1024.0, 70.0),
    (2048.0, 1036.0, 70.0),
    (-1788.0, 2300.0, 70.0),
    (1788.0, 2300.0, 70.0),
    (-3584.0, 2484.0, 70.0),
    (3584.0, 2484.0, 70.0),
    (0.0, 2816.0, 70.0),
    (- 940.0, 3310.0, 70.0),
    (940.0, 3308.0, 70.0),
    (-3072.0, 4096.0, 73.0),
    (3072.0, 4096.0, 73.0),
    (-1792.0, 4184.0, 70.0),
    (1792.0, 4184.0, 70.0),
    (0.0, 4240.0, 70.0),
)
